﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Reflection;

namespace Stoermeldungen.Class
{
    /// <summary>
    /// Die Grundklasse der Webinhaltsseiten
    /// </summary>
    public class PageBase : System.Web.UI.Page
    {
        public DataTable dt_Anzeige
        {
            get
            {
                return (DataTable)Session["dt_Anzeige"];
            }
            set
            {
                Session["dt_Anzeige"] = value;
            }
        }
        /// <summary>
        /// Länge des Angezeigten Fehlertextes in der Tabelle, bis
        /// es zu einer Kürzung kommt.
        /// </summary>
        public int LaengeFehlertext
        {
            get
            {
                try
                {

                    return Convert.ToInt32(ConfigurationSettings.AppSettings["Laenge_Fehlertexte"]);
                }
                catch
                {
                    return 40;
                }
            }
        }

        public int LaengeBMK
        {
            get
            {
                try
                {

                    return Convert.ToInt32(ConfigurationSettings.AppSettings["Laenge_BMK"]);
                }
                catch
                {
                    return 7;
                }
            }
        }

        /// <summary>
        /// Anzeige einer Nachricht als Alert Box
        /// </summary>
        /// <param name="sMessage">eine Nachricht (ohne '!)</param>
        public void ShowMessage(string sMessage)
        {
            if (sMessage.IndexOf("'") >= 0)
                sMessage = sMessage.Replace("'", "|");

            string sScript = "alert('" + sMessage + "')";
            ClientScript.RegisterStartupScript(this.GetType(),"", "<script>" + sScript + "</script>" );
        }

        /// <summary>
        /// Aufruf einer neuen Seite (die alte wird sich gemerkt)
        /// </summary>
        /// <param name="sNewPageURL">Die URL der neuen Seite</param>
        public void ChangePage(string sNewPageURL)
        {
            Session["LastPage"] = Request.Path;
            Response.Redirect(sNewPageURL, true);
        }

        /// <summary>
        /// Setzen des Seitennames im Banner
        /// </summary>
        /// <param name="sTitle">Der Seitenname</param>
        public void setTitle(string sTitle)
        {
            Label mpLabel = (Label)Master.FindControl("lb_Seite");
            if (mpLabel != null)
            {
                mpLabel.Text = sTitle;
            }
            return;
        }

        protected override void OnPreRender(EventArgs e)
        {
            // Verarbeitung der Benutzerrechte geschieht,
            //bevor die Seite an den Browser geschickt wird.

            DataTable dt;

            //Gecachte Daten benutzen, falls verfügbar.
            if(Cache["Benutzerrechte"] != null)
            {
                dt = (DataTable)Cache["Benutzerrechte"];
            }
            else
            {
                string sSql = @"SELECT * FROM T_Passwort_button";
                // Daten holen
                dt = DataConnection.ExecuteQuery(sSql);
                if(dt == null)
                {
                    ShowMessage("Fehler beim Zugriff auf die Datenbank: "+DataConnection.sErrorText);
                    return;
                }
                Cache.Insert("Benutzerrechte", dt,
                    null, System.DateTime.Now.AddMinutes(30),
                    TimeSpan.Zero, System.Web.Caching.CacheItemPriority.High, null);
            }

            int iBenutzerLevel;
            int iControlLevel;
            string sAktion;
            bool bRecht;

            // Abhängig vom Level des users werden einige Elemente besonders behandlt.
            if(Session["UserLevel"] == null)
                iBenutzerLevel = 0;
            else
            {
                try
                {
                    iBenutzerLevel = (int) Session["UserLevel"];
                }
                catch
                {
                    iBenutzerLevel = 0;
                }
            }
            foreach(Control theControl in Master.Controls)
            {                
                sAktion = theControl.ID;

            }
            foreach(DataRow row in dt.Rows)
            {
                //Angabe zum Zugriffsrecht vorhanden?
                try
                {
                    iControlLevel = Convert.ToInt32(row["benutzerlevel"]);
                }
                catch
                {
                    continue; //nein: ignorieren
                }
                sAktion = row["aktion"].ToString();

                if(iBenutzerLevel >= iControlLevel)
                    bRecht = true;
                else
                    bRecht = false;

                //Sonderfall HTML Element
                //Diese müssen per Javascript versteckt werden.
                //Dies trifft auf alle Elemente zu, die keine Webcontrols sind.
                if(row["Klasse"].ToString().ToLower() == "html")
                {
                    if(!bRecht)
                        ClientScript.RegisterStartupScript(this.GetType(), "BenutzerRecht_" + row["Buttonname"].ToString(),
                            "<script>ButtonsVerstecken('" + row["Buttonname"].ToString() + "')</script>");
                    continue;
                }

                // Control finden
                Control control = Master.FindControl("ContentPlaceHolder1");
                if(control == null)
                {
                    break; //Fataler Fehler: es gibt keine Chance mehr, die anderen Controls zu finden.             
                }
                control = control.FindControl(row["Buttonname"].ToString());
                if(control != null)
                {
                    if(sAktion.ToLower() == "enabled")
                    {
                        // Nicht jedes Control hat eine "Enabled" Eigenschaft...
                        Type t = control.GetType();
                        PropertyInfo info = t.GetProperty("Enabled");
                        if(info != null)
                            info.SetValue(control, bRecht, null);
                    }
                    else if(sAktion.ToLower() == "visible")                      
                        control.Visible = bRecht;                   
                }
            }
        }
    }

    /// <summary>
    /// Hilfsklasse für undefiniertes Datum.
    /// </summary>
    public class NotDefinedDate
    {
        private NotDefinedDate()
        {
        }

        public static DateTime Date
        {
            get
            {
                return (DateTime.Parse("01-01-1900"));
            }
        }
        public static string DateFormatForSelect = "yyyy-MM-dd HH:mm:ss";
    }
}
